// DCrypt ver. 1.00
// programmer: Ciprian Zavoianu
// contact: zavoianu.ciprian@info.uvt.ro
// please see License.txt file

// the DES encryption methods were written by Franois PIETTE in 2004
// for more information regarding DES methods: http://www.koders.com/delphi/fid82119510EA84E4057CDC9786AEFC59189CA82327.aspx?s=DES#L6
unit DelphiCryptography;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Buttons, Mask;

type
  TPolybiusSquare = array [0..35] of byte;
  PArrayOf8Bytes  = ^TArrayOf8Bytes;
  TArrayOf8Bytes  = array [0..7] of Byte;
  TArrayOf16Bytes = array [1..16] of Byte;
  TArrayOf28Bytes = array [1..28] of Byte;
  TArrayOf32Bytes = array [1..32] of Byte;
  TArrayOf48Bytes = array [1..48] of Byte;
  TArrayOf56Bytes = array [1..56] of Byte;
  TArrayOf64Bytes = array [1..64] of Byte;

  TDesData = record
    InputValue     : TArrayOf64Bytes;
    OutputValue    : TArrayOf64Bytes;
    RoundKeys      : array [1..16] of TArrayOf48Bytes;
    L, R           : TArrayOf32Bytes;
    FunctionResult : TArrayOf32Bytes;
    C, D           : TArrayOf28Bytes;
  end;

  TfrmMain = class(TForm)
    pnTop: TPanel;
    pnBottom: TPanel;
    pnLeft: TPanel;
    pnRight: TPanel;
    pnCenter: TPanel;
    Input: TGroupBox;
    gbOutput: TGroupBox;
    MemoInput: TMemo;
    MemoOutput: TMemo;
    gbVigenere: TGroupBox;
    medPass1: TMaskEdit;
    lbInfo1: TLabel;
    btEncode1: TBitBtn;
    btDecode1: TBitBtn;
    cbVigenere: TComboBox;
    Label1: TLabel;
    gbBifid: TGroupBox;
    lbInfo3: TLabel;
    medPass2: TMaskEdit;
    btEncode2: TBitBtn;
    btDecode2: TBitBtn;
    GroupBox1: TGroupBox;
    btEncode3: TBitBtn;
    btDecode3: TBitBtn;
    Label2: TLabel;
    btRedirect: TBitBtn;
    medPass3: TMaskEdit;
    procedure btEncode1Click(Sender: TObject);
    procedure btDecode1Click(Sender: TObject);
    procedure btEncode2Click(Sender: TObject);
    procedure btDecode2Click(Sender: TObject);
    procedure btEncode3Click(Sender: TObject);
    procedure btDecode3Click(Sender: TObject);
    procedure btRedirectClick(Sender: TObject);
  private
    procedure VigenereEncode (AnInput, AKey : string; const NrOfTurns : integer; var AnOutput : string);
    procedure VigenereDecode (AnInput, AKey : string; const NrOfTurns : integer; var AnOutput : string);
    procedure BifidEncode (AnInput, AKey : string; const NrOfTurns : integer; var AnOutput : string);
    procedure BifidDecode (AnInput, AKey : string; const NrOfTurns : integer; var AnOutput : string);
    procedure DESEncode (AnInput, AKey : string; var AnOutput : string);
    procedure DESDecode (AnInput, AKey : string; var AnOutput : string);
  public

  end;

var
  frmMain: TfrmMain;

const
    { Initial Permutation }
    IP : TArrayOf64Bytes =      (58, 50, 42, 34, 26, 18, 10, 2, 
                                 60, 52, 44, 36, 28, 20, 12, 4, 
                                 62, 54, 46, 38, 30, 22, 14, 6, 
                                 64, 56, 48, 40, 32, 24, 16, 8,
                                 57, 49, 41, 33, 25, 17,  9, 1,
                                 59, 51, 43, 35, 27, 19, 11, 3,
                                 61, 53, 45, 37, 29, 21, 13, 5,
                                 63, 55, 47, 39, 31, 23, 15, 7);
    { Final Permutation }
    InvIP : TArrayOf64Bytes =   (40,  8, 48, 16, 56, 24, 64, 32,
                                 39,  7, 47, 15, 55, 23, 63, 31,
                                 38,  6, 46, 14, 54, 22, 62, 30,
                                 37,  5, 45, 13, 53, 21, 61, 29,
                                 36,  4, 44, 12, 52, 20, 60, 28,
                                 35,  3, 43, 11, 51, 19, 59, 27,
                                 34,  2, 42, 10, 50, 18, 58, 26,
                                 33,  1, 41,  9, 49, 17, 57, 25);
    { Expansion Permutation }
    E : TArrayOf48Bytes =       (32,  1,  2,  3,  4,  5,
                                  4,  5,  6,  7,  8,  9,
                                  8,  9, 10, 11, 12, 13,
                                 12, 13, 14, 15, 16, 17,
                                 16, 17, 18, 19, 20, 21,
                                 20, 21, 22, 23, 24, 25,
                                 24, 25, 26, 27, 28, 29,
                                 28, 29, 30, 31, 32,  1);
    { P-Box permutation }
    P : TArrayOf32Bytes =       (16,  7, 20, 21, 29, 12, 28, 17,
                                  1, 15, 23, 26,  5, 18, 31, 10,
                                  2,  8, 24, 14, 32, 27,  3,  9,
                                 19, 13, 30,  6, 22, 11,  4, 25);
    { Key Permutation }
    PC_1 : TArrayOf56Bytes =    (57, 49, 41, 33, 25, 17,  9,
                                  1, 58, 50, 42, 34, 26, 18,
                                 10,  2, 59, 51, 43, 35, 27,
                                 19, 11,  3, 60, 52, 44, 36,
                                 63, 55, 47, 39, 31, 23, 15,
                                  7, 62, 54, 46, 38, 30, 22,
                                 14,  6, 61, 53, 45, 37, 29,
                                 21, 13,  5, 28, 20, 12,  4);
    { Compression Permutation }
    PC_2 : TArrayOf48Bytes =    (14, 17, 11, 24,  1,  5,
                                  3, 28, 15,  6, 21, 10,
                                 23, 19, 12,  4, 26,  8,
                                 16,  7, 27, 20, 13,  2,
                                 41, 52, 31, 37, 47, 55,
                                 30, 40, 51, 45, 33, 48,
                                 44, 49, 39, 56, 34, 53,
                                 46, 42, 50, 36, 29, 32);
    { Number of key bits shifted per round }
    ST : TArrayOf16Bytes =      ( 1,  1,  2,  2,  2,  2,  2,  2,
                                  1,  2,  2,  2,  2,  2,  2,  1);
    { S-Boxes }
    SBoxes : array [1..8,  0..3,  0..15] of Byte =
          (((14,  4, 13,  1,  2, 15, 11,  8,  3, 10,  6, 12,  5,  9,  0,  7),
            ( 0, 15,  7,  4, 14,  2, 13,  1, 10,  6, 12, 11,  9,  5,  3,  8),
            ( 4,  1, 14,  8, 13,  6,  2, 11, 15, 12,  9,  7,  3, 10,  5,  0),
            (15, 12,  8,  2,  4,  9,  1,  7,  5, 11,  3, 14, 10,  0,  6, 13)),

            ((15,  1,  8, 14,  6, 11,  3,  4,  9,  7,  2, 13, 12,  0,  5, 10),
            ( 3, 13,  4,  7, 15,  2,  8, 14, 12,  0,  1, 10,  6,  9, 11,  5),
            ( 0, 14,  7, 11, 10,  4, 13,  1,  5,  8, 12,  6,  9,  3,  2, 15),
            (13,  8, 10,  1,  3, 15,  4,  2, 11,  6,  7, 12,  0,  5, 14,  9)),

            ((10,  0,  9, 14,  6,  3, 15,  5,  1, 13, 12,  7, 11,  4,  2,  8),
            (13,  7,  0,  9,  3,  4,  6, 10,  2,  8,  5, 14, 12, 11, 15,  1),
            (13,  6,  4,  9,  8, 15,  3,  0, 11,  1,  2, 12,  5, 10, 14,  7),
            ( 1, 10, 13,  0,  6,  9,  8,  7,  4, 15, 14,  3, 11,  5,  2, 12)),

            ((7, 13, 14,  3,  0,  6,  9, 10,  1,  2,  8,  5, 11, 12,  4, 15),
            (13,  8, 11,  5,  6, 15,  0,  3,  4,  7,  2, 12,  1, 10, 14,  9),
            (10,  6,  9,  0, 12, 11,  7, 13, 15,  1,  3, 14,  5,  2,  8,  4),
            ( 3, 15,  0,  6, 10,  1, 13,  8,  9,  4,  5, 11, 12,  7,  2, 14)),

            ((2, 12,  4,  1,  7, 10, 11,  6,  8,  5,  3, 15, 13,  0, 14,  9),
            (14, 11,  2, 12,  4,  7, 13,  1,  5,  0, 15, 10,  3,  9,  8,  6),
            ( 4,  2,  1, 11, 10, 13,  7,  8, 15,  9, 12,  5,  6,  3,  0, 14),
            (11,  8, 12,  7,  1, 14,  2, 13,  6, 15,  0,  9, 10,  4,  5,  3)),

            ((12,  1, 10, 15,  9,  2,  6,  8,  0, 13,  3,  4, 14,  7,  5, 11),
            (10, 15,  4,  2,  7, 12,  9,  5,  6,  1, 13, 14,  0, 11,  3,  8),
            ( 9, 14, 15,  5,  2,  8, 12,  3,  7,  0,  4, 10,  1, 13, 11,  6),
            ( 4,  3,  2, 12,  9,  5, 15, 10, 11, 14,  1,  7,  6,  0,  8, 13)),

            ((4, 11,  2, 14, 15,  0,  8, 13,  3, 12,  9,  7,  5, 10,  6,  1),
            (13,  0, 11,  7,  4,  9,  1, 10, 14,  3,  5, 12,  2, 15,  8,  6),
            ( 1,  4, 11, 13, 12,  3,  7, 14, 10, 15,  6,  8,  0,  5,  9,  2),
            ( 6, 11, 13,  8,  1,  4, 10,  7,  9,  5,  0, 15, 14,  2,  3, 12)),

            ((13,  2,  8,  4,  6, 15, 11,  1, 10,  9,  3, 14,  5,  0, 12,  7),
            ( 1, 15, 13,  8, 10,  3,  7,  4, 12,  5,  6, 11,  0, 14,  9,  2),
            ( 7, 11,  4,  1,  9, 12, 14,  2,  0,  6, 10, 13, 15,  3,  5,  8),
            ( 2,  1, 14,  7,  4, 10,  8, 13, 15, 12,  9,  0,  3,  5,  6, 11)));


implementation

{$R *.dfm}

function HexToInt(s: string): integer;
begin
  Result := StrToInt('$' + s);
end;

function TrimAllChar(const S: string; const ch: Char): string;
var
  buf: string;
begin
  buf := S;
  Result := '';
  {while Pos finds a blank}
  while (Pos(ch, buf) > 0) do
    begin
      Result := Result + Copy(buf, 1, Pos(ch, buf) - 1);
      buf := Copy(buf, Pos(ch, buf) + 1, Length(buf) - Pos(ch, buf));
    end;
  Result := Result + buf;
end;

function GetBit(const Bits: TArrayOf8Bytes; Index: Byte): Byte;
begin
    Dec(Index);
    if Bits[Index div 8] and (128 shr(Index mod 8)) > 0 then
        Result := 1
    else
        Result := 0;
end;

procedure SetBit(var Bits: TArrayOf8Bytes; Index, Value : Byte);
var
  Bit: Byte;
begin
    Dec(Index);
    Bit := 128 shr(Index mod 8);
    case Value of
    0: Bits[Index div 8] := Bits[Index div 8] and (not Bit);
    1: Bits[Index div 8] := Bits[Index div 8] or Bit;
    end;
end;

procedure F(var FR: TArrayOf32Bytes; var FK: TArrayOf48Bytes; var TotalOut: TArrayOf32Bytes);
var
  Aux1 : TArrayOf48Bytes;
  Aux2 : TArrayOf32Bytes;
  n    : integer;   
  h    : integer;
  i    : integer;
  j    : integer;
  Line : integer;
  Col  : integer;
begin
    for n := 1 to 48 do
        Aux1[n] := FR[E[n]] xor FK[n];
    for n := 1 to 8 do begin
        i := (n - 1) * 6;
        j := (n - 1) * 4;
        Line := Aux1[i + 1] * 2 + Aux1[i + 6];
        Col := Aux1[i + 2] * 8 + Aux1[i + 3] * 4 + Aux1[i + 4] * 2 + Aux1[i + 5];
        for h := 1 to 4 Do begin
            case h of
            1: Aux2[j + h] := (SBoxes[n, Line, Col] and 8) div 8;
            2: Aux2[j + h] := (SBoxes[n, Line, Col] and 4) div 4;
            3: Aux2[j + h] := (SBoxes[n, Line, Col] and 2) div 2;
            4: Aux2[j + h] := (SBoxes[n, Line, Col] and 1);
            end;
        end;
    end;
    for n := 1 to 32 do
        TotalOut[n] := Aux2[P[n]];
end;

procedure Shift(var SubKeyPart : TArrayOf28Bytes);
var
  n: byte;
  b: byte;
begin
    b := SubKeyPart[1];
    for n := 1 to 27 do
        SubKeyPart[n] := SubKeyPart[n + 1];
    SubKeyPart[28] := b;
end;

procedure SubKey(var DesData: TDesData; Round: byte; var SubKey: TArrayOf48Bytes);
var
  n : Byte;
  b : Byte;
begin
    for n := 1 to ST[Round] do begin
        Shift(DesData.C);
        Shift(DesData.D);
    end;
    for n := 1 to 48 do begin
        b := PC_2[n];
        if b <= 28 then
            SubKey[n] := DesData.C[b]
        else
            SubKey[n] := DesData.D[b - 28];
    end;
end;

procedure DES(Input, Key: TArrayOf8Bytes; var Output: TArrayOf8Bytes; Encrypt: Boolean);
var
  n : byte;
  b : byte;
  Round : byte;
  DesData : TDesData;
begin
    for n := 1 to 64 do
        DesData.InputValue[n] := GetBit(Input, n);
    for n := 1 to 28 do begin
        DesData.C[n] := GetBit(Key, PC_1[n]);
        DesData.D[n] := GetBit(Key, PC_1[n + 28]);
    end;
    for n := 1 to 16 do
        SubKey(DesData, n, DesData.RoundKeys[n]);
    for n := 1 to 64 do begin
        if n <= 32 then
            DesData.L[n]      := DesData.InputValue[IP[n]]
        else
            DesData.R[n - 32] := DesData.InputValue[IP[n]];
    end;
    for Round := 1 to 16 do begin
        if Encrypt then
            F(DesData.R, DesData.RoundKeys[Round], DesData.FunctionResult)
        else
            F(DesData.R, DesData.RoundKeys[17 - Round], DesData.FunctionResult);
        for n := 1 to 32 do
            DesData.FunctionResult[n] := DesData.FunctionResult[n] xor DesData.L[n];
        DesData.L := DesData.R;
        DesData.R := DesData.FunctionResult;
    end;
    for n := 1 to 64 do begin
        b := InvIP[n];
        if b <= 32 then
            DesData.OutputValue[n] := DesData.R[b]
        else
            DesData.OutputValue[n] := DesData.L[b-32];
    end;
    for n := 1 to 64 do
        SetBit(Output, n, DesData.OutputValue[n]);
end;

procedure ConstructPolybiusSquare (AKey: string; var APolybiusSquare : TPolybiusSquare);
var
  Marker   : TPolybiusSquare;
  KeyValue : byte;
  i        : integer;
  pi       : integer;
  s        : string;
begin
  for i := 0 to 35 do
    begin
      Marker[i] := 0;
    end;
  i := 1;
  pi := -1;
  while i <= Length(AKey) do
    begin
      KeyValue := Ord(AKey[i]) - 65;
      if Marker[KeyValue] = 0 then
        begin
          Inc(pi);
          APolybiusSquare[pi] := Ord(AKey[i]);
          Marker[KeyValue] := 1;
        end;
      Inc(i);
    end;

  for i := 90 downto 65 do
    begin
      KeyValue := i - 65;
      if Marker[KeyValue] = 0 then
        begin
          Inc(pi);
          APolybiusSquare[pi] := Ord(i);
          Marker[KeyValue] := 1;
        end;
    end;

  APolybiusSquare[26] := Ord(':');
  APolybiusSquare[27] := Ord('-');
  APolybiusSquare[28] := Ord(' ');
  APolybiusSquare[29] := 13;
  APolybiusSquare[30] := 10;
  APolybiusSquare[31] := Ord('!');
  APolybiusSquare[32] := Ord('.');
  APolybiusSquare[33] := Ord(',');
  APolybiusSquare[34] := Ord(';');
  APolybiusSquare[35] := Ord('?');

  {s := '';
  for i := 0 to 35 do
    s := s + Chr(APolybiusSquare[i]) + ' ';
  ShowMessage(s);}
end;

procedure TfrmMain.DESEncode(AnInput, AKey: string; var AnOutput: string);
var
  DESIn  : TArrayOf8Bytes;
  DESOut : TArrayOf8Bytes;
  Key    : TArrayOf8Bytes;
  i      : integer;
  j      : integer;
  diff   : integer;
begin
  for i := 1 to 8 do
    Key[i-1] := Ord(AKey[i]);
  diff := Length(AnInput) mod 8;
  if diff <> 0 then
    for i := 1 to (8 - diff) do
      AnInput := AnInput + ' ';
  i := 1;
  while i < Length(AnInput) do
    begin
      DESIn[0] := Ord(AnInput[i+0]);
      DESIn[1] := Ord(AnInput[i+1]);
      DESIn[2] := Ord(AnInput[i+2]);
      DESIn[3] := Ord(AnInput[i+3]);
      DESIn[4] := Ord(AnInput[i+4]);
      DESIn[5] := Ord(AnInput[i+5]);
      DESIn[6] := Ord(AnInput[i+6]);
      DESIn[7] := Ord(AnInput[i+7]);

      for j := 0 to 7 do
        DESOut[j] := 0;
        
      DES(DESIn, Key, DESOut, true);

      AnOutput := AnOutput + Chr(DESOut[0]);
      AnOutput := AnOutput + Chr(DESOut[1]);
      AnOutput := AnOutput + Chr(DESOut[2]);
      AnOutput := AnOutput + Chr(DESOut[3]);
      AnOutput := AnOutput + Chr(DESOut[4]);
      AnOutput := AnOutput + Chr(DESOut[5]);
      AnOutput := AnOutput + Chr(DESOut[6]);
      AnOutput := AnOutput + Chr(DESOut[7]);
      i := i + 8;
    end;
end;

procedure TfrmMain.DESDecode(AnInput, AKey: string; var AnOutput: string);
var
  DESIn  : TArrayOf8Bytes;
  DESOut : TArrayOf8Bytes;
  Key    : TArrayOf8Bytes;
  i      : integer;
  j      : integer;
  diff   : integer;
begin
  for i := 1 to 8 do
    Key[i-1] := Ord(AKey[i]);
  diff := Length(AnInput) div 8;
  {if diff <> 0 then
    for i := 1 to (8 - diff) do
      AnInput := AnInput + ' ';}
  i := 1;
  while i < Length(AnInput) do
    begin
      DESIn[0] := Ord(AnInput[i+0]);
      DESIn[1] := Ord(AnInput[i+1]);
      DESIn[2] := Ord(AnInput[i+2]);
      DESIn[3] := Ord(AnInput[i+3]);
      DESIn[4] := Ord(AnInput[i+4]);
      DESIn[5] := Ord(AnInput[i+5]);
      DESIn[6] := Ord(AnInput[i+6]);
      DESIn[7] := Ord(AnInput[i+7]);

      for j := 0 to 7 do
        DESOut[j] := 0;

      DES(DESIn, Key, DESOut, false);

      AnOutput := AnOutput + Chr(DESOut[0]);
      AnOutput := AnOutput + Chr(DESOut[1]);
      AnOutput := AnOutput + Chr(DESOut[2]);
      AnOutput := AnOutput + Chr(DESOut[3]);
      AnOutput := AnOutput + Chr(DESOut[4]);
      AnOutput := AnOutput + Chr(DESOut[5]);
      AnOutput := AnOutput + Chr(DESOut[6]);
      AnOutput := AnOutput + Chr(DESOut[7]);
      i := i + 8;
    end;
end;

procedure FindCoord( AChar : char; APSquare : TPolybiusSquare; var ALine, ACol : byte);
var
  found : boolean;
  i     : integer;
begin
  for i := 0 to 35 do
    if APSquare[i] = Ord(AChar) then
      break;
  ALine := i div 6;
  ACol := i mod 6;
end;

function GetChar(APSquare : TPolybiusSquare; Line, Col : integer) : char;
begin
  Result := Chr(APSquare[Line * 6 + Col]);
end;  

procedure TfrmMain.BifidEncode (AnInput, AKey : string; const NrOfTurns : integer; var AnOutput : string);
var
  PSquare : TPolybiusSquare;
  Line    : byte;
  Col     : byte;
  Vector  : array of byte;
  i       : integer;
  vi      : integer;
  s        : string;
begin
  ConstructPolybiusSquare(AKey, PSquare);
  SetLength(Vector, Length(AnInput)*2);
  vi := -1;
  for i := 1 to Length(AnInput) do
    begin
      FindCoord(AnInput[i], PSquare, Line, Col);
      Inc(vi);
      Vector[vi] := Line;
      Vector[vi + Length(AnInput)] := Col;
    end;
  i := 0;
  while i <= Length(AnInput)*2 -1 do
    begin
      AnOutput := AnOutput + GetChar(PSquare, Vector[i], Vector[i+1]);
      i := i + 2;
    end;
end;

procedure TfrmMain.BifidDecode (AnInput, AKey : string; const NrOfTurns : integer; var AnOutput : string);
var
  PSquare : TPolybiusSquare;
  Line    : byte;
  Col     : byte;
  Vector  : array of byte;
  i       : integer;
  vi      : integer;
  s        : string;
begin
  ConstructPolybiusSquare(AKey, PSquare);
  SetLength(Vector, Length(AnInput)*2);
  vi := -1;
  for i := 1 to Length(AnInput) do
    begin
      FindCoord(AnInput[i], PSquare, Line, Col);
      Inc(vi);
      Vector[vi] := Line;
      Inc(vi);
      Vector[vi] := Col;
    end;

  i := 0;
  while i < Length(AnInput) do
    begin
      AnOutput := AnOutput + GetChar(PSquare, Vector[i], Vector[i+Length(AnInput)]);
      Inc(i);
    end;
end;

procedure TfrmMain.VigenereEncode (AnInput, AKey : string; const NrOfTurns : integer;  var AnOutput : string);
var
  i         : integer;
  j         : integer;
  KeyLength : integer;
  KeyIndex  : integer;
  Offset    : byte;
  NewChar   : char;
begin
  for j := 1 to NrOfTurns do
    begin
      KeyLength := Length(AKey);
      KeyIndex := 1;
      for i := 1 to Length(AnInput) do
        if (Ord(AnInput[i]) >= 65) and (Ord(AnInput[i]) <= 90) then
          begin
            Offset := Ord(AKey[KeyIndex]) - 65;
            NewChar := Chr(Ord(AnInput[i]) + Offset);

            if Ord(NewChar) > 90 then
              NewChar := Char(64 + (Ord(NewChar) - 90));

            AnOutput := AnOutput + NewChar;

            Inc(KeyIndex);
            if KeyIndex > KeyLength then
              KeyIndex := 1;
          end
        else
          AnOutput := AnOutput + AnInput[i];
      AnInput := AnOutput;
      AnOutput := '';    
    end;
  AnOutput := AnInput;  
end;

procedure TfrmMain.VigenereDecode (AnInput, AKey : string; const NrOfTurns : integer; var AnOutput : string);
var
  i         : integer;
  j         : integer;
  KeyLength : integer;
  KeyIndex  : integer;
  Offset    : byte;
  NewChar   : char;
begin
  for j := 1 to NrOfTurns do
    begin
      KeyLength := Length(AKey);
      KeyIndex := 1;
      for i := 1 to Length(AnInput) do
        if (Ord(AnInput[i]) >= 65) and (Ord(AnInput[i]) <= 90) then
          begin
            Offset := Ord(AKey[KeyIndex]) - 65;
            NewChar := Chr(Ord(AnInput[i]) - Offset);

            if Ord(NewChar) < 65 then
              NewChar := Char(91 - (65 - Ord(NewChar)));

            AnOutput := AnOutput + NewChar;

            Inc(KeyIndex);
            if KeyIndex > KeyLength then
              KeyIndex := 1;
          end
        else
          AnOutput := AnOutput + AnInput[i];
      AnInput := AnOutput;
      AnOutput := '';
    end;
  AnOutput := AnInput;
end;

procedure TfrmMain.btDecode1Click(Sender: TObject);
var
  OriginalText : string;
  DecodedText  : string;
  s            : string;
  i            : integer;
begin
  s := UpperCase(MemoInput.Lines.Text);
  s := TrimAllChar(s, #10);
  s := TrimAllChar(s, #13);
  OriginalText := '';
  i := 1;
  while i <= Length(s) do
    begin
      OriginalText := OriginalText + Chr(HexToInt('' + s[i] + s[i+1]));
      i := i+2;
    end;
  VigenereDecode(OriginalText, medPass1.Text, StrToInt(cbVigenere.Items[cbVigenere.ItemIndex]),  DecodedText);
  MemoOutput.Lines.Clear;
  MemoOutput.Lines.Append(DecodedText);
end;

procedure TfrmMain.btDecode2Click(Sender: TObject);
var
  OriginalText : string;
  DecodedText  : string;
  s            : string;
  i            : integer;
begin
  s := UpperCase(MemoInput.Lines.Text);
  s := TrimAllChar(s, #10);
  s := TrimAllChar(s, #13);
  OriginalText := '';
  i := 1;
  while i <= Length(s) do
    begin
      OriginalText := OriginalText + Chr(HexToInt('' + s[i] + s[i+1]));
      i := i+2;
    end;
  BifidDecode(OriginalText, medPass2.Text, 1,  DecodedText);
  MemoOutput.Lines.Clear;
  MemoOutput.Lines.Append(DecodedText);
end;

procedure TfrmMain.btDecode3Click(Sender: TObject);
var
  OriginalText : string;
  EncodedText  : string;
  Key          : string;
  s            : string;
  i            : integer;
  l            : integer;
begin
  Key := '';
  s := medPass3.Text;
  l := Length(s);
  for i := 1 to 8-l do
    s := s + '*';
  Key := s[1] + s[l] + s[2] + s[l-1] +
         s[l-2] + s[4] + s[3] + s[l-3];

  s := MemoInput.Lines.GetText;
  s := TrimAllChar(s, #10);
  s := TrimAllChar(s, #13);
  OriginalText := '';
  i := 1;
  while i <= Length(s) do
    begin
      OriginalText := OriginalText + Chr(HexToInt('' + s[i] + s[i+1]));
      i := i+2;
    end;
  DesDecode(OriginalText, Key, EncodedText);
  MemoOutput.Lines.Clear;
  MemoOutput.Lines.Append(EncodedText);
end;

procedure TfrmMain.btEncode1Click(Sender: TObject);
var
  OriginalText : string;
  EncodedText  : string;
  i            : integer;
  s            : string;
begin
  OriginalText := UpperCase(MemoInput.Lines.Text);
  VigenereEncode(OriginalText, medPass1.Text, StrToInt(cbVigenere.Items[cbVigenere.ItemIndex]),  EncodedText);
  s := '';
  for i := 1 to Length(EncodedText) do
    s := s + IntToHex(Ord(EncodedText[i]),2);
  MemoOutput.Lines.Clear;
  MemoOutput.Lines.Append(s);
end;

procedure TfrmMain.btEncode2Click(Sender: TObject);
var
  OriginalText : string;
  EncodedText  : string;
  i            : integer;
  s            : string;
begin
  OriginalText := UpperCase(MemoInput.Lines.Text);
  BifidEncode(OriginalText, medPass2.Text, 1,  EncodedText);
  s := '';
  for i := 1 to Length(EncodedText) do
    s := s + IntToHex(Ord(EncodedText[i]),2);
  MemoOutput.Lines.Clear;
  MemoOutput.Lines.Append(s);
end;

procedure TfrmMain.btEncode3Click(Sender: TObject);
var
  OriginalText : string;
  EncodedText  : string;
  Key          : string;
  i            : integer;
  s            : string;
  l            : integer;
begin
  Key := '';
  s := medPass3.Text;
  l := Length(s);
  for i := 1 to 8-l do
    s := s + '*';
  Key := s[1] + s[l] + s[2] + s[l-1] +
         s[l-2] + s[4] + s[3] + s[l-3];

  OriginalText := MemoInput.Lines.Text;
  DesEncode(OriginalText, Key, EncodedText);
  s := '';
  for i := 1 to Length(EncodedText) do
    s := s + IntToHex(Ord(EncodedText[i]),2);
  MemoOutput.Lines.Clear;
  MemoOutput.Lines.Append(s);
end;

procedure TfrmMain.btRedirectClick(Sender: TObject);
begin
  MemoInput.Lines.Clear;
  MemoInput.Text := MemoOutput.Text;
end;

end.
